/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                    Java Language Motus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* GPIO Configuration */
public class ANPacket188
{
	public static final int LENGTH = 13;

	
	/* Please refer to the manual to know which GPIO functions are available for each port */
	public static final int MODE_INACTIVE					= 0;
	public static final int MODE_1PPS_OUTPUT				= 1;
	public static final int MODE_GNSS_FIX_OUTPUT			= 2;
	public static final int MODE_ODOMETER_INPUT				= 3;
	public static final int MODE_ZERO_VELOCITY_INPUT		= 4;
	public static final int MODE_PITOT_TUBE_INPUT			= 5;
	public static final int MODE_NMEA0183_INPUT				= 6;
	public static final int MODE_NMEA0183_OUTPUT			= 7;
	public static final int MODE_NOVATEL_GNSS_INPUT			= 8;
	public static final int MODE_TOPCON_GNSS_INPUT			= 9;
	public static final int MODE_ANPP_INPUT					= 11;
	public static final int MODE_ANPP_OUTPUT				= 12;
	public static final int MODE_DISABLE_MAGNETOMETERS		= 13;
	public static final int MODE_DISABLE_GNSS				= 14;
	public static final int MODE_DISABLE_PRESSURE			= 15;
	public static final int MODE_SET_ZERO_ORIENTATION_ALIGN	= 16;
	public static final int MODE_SYSTEM_STATE_PACKET_TRIGGER= 17;
	public static final int MODE_RAW_SENSORS_PACKET_TRIGGER	= 18;
	public static final int MODE_RTCM_CORRECTIONS_INPUT		= 19;
	public static final int MODE_TRIMBLE_GNSS_INPUT			= 20;
	public static final int MODE_UBLOX_GNSS_INPUT			= 21;
	public static final int MODE_HEMISPHERE_GNSS_INPUT		= 22;
	public static final int MODE_TELEDYNE_DVL_INPUT			= 23;
	public static final int MODE_TRITECH_USB_INPUT			= 24;
	public static final int MODE_LINKQUEST_DVL_INPUT		= 25;
	public static final int MODE_PRESSURE_DEPTH_TRANSDUCER	= 26;
	public static final int MODE_LEFT_WHEEL_SPEED_SENSOR	= 27;
	public static final int MODE_RIGHT_WHEEL_SPEED_SENSOR	= 28;
	public static final int MODE_1PPS_INPUT					= 29;
	public static final int MODE_WHEEL_SPEED_SENSOR			= 30;
	public static final int MODE_WHEEL_ENCODER_PHASE_A		= 31;
	public static final int MODE_WHEEL_ENCODER_PHASE_B		= 32;
	public static final int MODE_EVENT_1_INPUT				= 33;
	public static final int MODE_EVENT_2_INPUT				= 34;
	public static final int MODE_LINKQUEST_USBL_INPUT		= 35;
	public static final int MODE_GNSS_RECEIVER_PASSTHROUGH	= 38;
	public static final int MODE_TSS1_OUTPUT				= 39;
	public static final int MODE_SIMRAD1000_OUTPUT			= 40;
	public static final int MODE_SIMRAD3000_OUTPUT			= 41;
	public static final int MODE_SERIAL_PORT_PASSTHROUGH	= 42;
	public static final int MODE_GIMBAL_ENCODER_PHASE_A		= 43;
	public static final int MODE_GIMBAL_ENCODER_PHASE_B		= 44;
	public static final int MODE_ODOMETER_DIRECTION_FWD_LOW	= 45;
	public static final int MODE_ODOMETER_DIRECTION_FWD_HIGH= 46;
	public static final int MODE_NORTEK_DVL_INPUT			= 51;
	public static final int MODE_MOVING_BASE_CORR_OUTPUT	= 52;
	public static final int MODE_REVERSE_ALIGNMENT_FWD_LOW	= 53;
	public static final int MODE_REVERSE_ALIGNMENT_FWD_HIGH	= 54;
	public static final int MODE_ZERO_ANGULAR_VELOCITY_INPUT= 55;
	public static final int MODE_MAVLINK_OUTPUT				= 56;

	public static final String[] GPIO_FUNCTIONS = 
	{
		"Inactive",								// 0
		"1PPS Output",							// 1
		"GNSS Fix Output",						// 2
		"Odometer Input",						// 3
		"Zero Velocity Input",					// 4
		"Pitot Tube Input",						// 5
		"NMEA Input",							// 6
		"NMEA Output",							// 7
		"Novatel GNSS Input",					// 8
		"Topcon GNSS Input",					// 9
		"Reserved",								// 10
		"ANPP Input",							// 11
		"ANPP Output",							// 12
		"Disable Magnetometers",				// 13
		"Disable GNSS",							// 14
		"Disable Pressure",						// 15
		"Set Zero Orientation Alignment",		// 16
		"System State Packet Trigger",			// 17
		"Raw Sensors Packet Trigger",			// 18
		"RTCM Differential Corrections Input",	// 19
		"Trimble GNSS Input",					// 20
		"u-blox GNSS Input",					// 21
		"Hemisphere GNSS Input",				// 22
		"Teledyne DVL Input",					// 23
		"Tritech USBL Input",					// 24
		"Linkquest DVL Input",					// 25
		"Pressure Depth Transducer",			// 26
		"Left Wheel Speed Sensor",				// 27
		"Right Wheel Speed Sensor",				// 28
		"1PPS Input",							// 29
		"Wheel Speed Sensor",					// 30
		"Wheel Encoder Phase A",				// 31
		"Wheel Encoder Phase B",				// 32
		"Event 1 Input",						// 33
		"Event 2 Input",						// 34
		"Linkquest USBL Input",					// 35
		"Reserved",								// 36
		"Reserved",								// 37
		"GNSS Receiver Passthrough",			// 38
		"TSS1 Output",							// 39
		"Simrad 1000 Output",					// 40
		"Simrad 3000 Output",					// 41
		"Serial Port Passthrough",				// 42
		"Gimbal Phase A",						// 43
		"Gimbal Phase B",						// 44
		"Odometer Direction, Forward Low",		// 45
		"Odometer Direction, Forward High",		// 46
		"Reserved",								// 47
		"Reserved",								// 48
		"Reserved",								// 49
		"Reserved",								// 50
		"Nortek DVL Input",						// 51
		"Moving Base Corrections Output",		// 52
		"Reverse Alignment, Forward Low",		// 53
		"Reverse Alignment, Forward High",		// 54
		"Zero Angular Velocity Input",			// 55
		"Mavlink Output"						// 56
	};

	public Boolean permanent;
	public int gpio1Function;
	public int gpio2Function;
	public int auxiliaryRS232TransmitFunction;
	public int auxiliaryRS232ReceiveFunction;

	public ANPacket188()
	{
	}

	public ANPacket188(ANPacket packet)
	{
		permanent = packet.data[0] != 0;
		gpio1Function = packet.data[1] & 0xFF;
		gpio2Function = packet.data[2] & 0xFF;
		auxiliaryRS232TransmitFunction = packet.data[3] & 0xFF;
		auxiliaryRS232ReceiveFunction = packet.data[4] & 0xFF;
	}

	public ANPacket toANPacket()
	{	
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_GPIO_CONFIGURATION);
		packet.data[0] = (byte) (permanent ? 1 : 0);
		packet.data[1] = (byte) gpio1Function;
		packet.data[2] = (byte) gpio2Function;
		packet.data[3] = (byte) auxiliaryRS232TransmitFunction;
		packet.data[4] = (byte) auxiliaryRS232ReceiveFunction;
		return packet;
	}

	public String gpio1FunctionString()
	{
		if(gpio1Function < GPIO_FUNCTIONS.length)
		{
			return GPIO_FUNCTIONS[gpio1Function];
		}
		else return "Unknown";
	}

	public String gpio2FunctionString()
	{
		if(gpio2Function < GPIO_FUNCTIONS.length)
		{
			return GPIO_FUNCTIONS[gpio2Function];
		}
		else return "Unknown";
	}

	public String auxiliaryRS232TransmitFunctionString()
	{
		if(auxiliaryRS232TransmitFunction < GPIO_FUNCTIONS.length)
		{
			return GPIO_FUNCTIONS[auxiliaryRS232TransmitFunction];
		}
		else return "Unknown";
	}

	public String auxiliaryRS232ReceiveFunctionString()
	{
		if(auxiliaryRS232ReceiveFunction < GPIO_FUNCTIONS.length)
		{
			return GPIO_FUNCTIONS[auxiliaryRS232ReceiveFunction];
		}
		else return "Unknown";
	}

	public static String portModeToString(int mode)
	{
		if (mode >= 0 && mode < GPIO_FUNCTIONS.length)
		{
			return GPIO_FUNCTIONS[mode];
		}
		return "Unknown";
	}
}
