/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                    Java Language Motus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anppExample;

import java.awt.EventQueue;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.DefaultComboBoxModel;

import com.advancednavigation.anPackets.*;
import com.fazecast.jSerialComm.*;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;

import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.JLabel;
import javax.swing.JTextField;

public class Main
{
	// GUI Elements
	private JFrame frmAnppExample;
	private JComboBox<String> comboSerialPort;
	private JComboBox<String> comboBaudRate;
	private JButton btnSerialConnect;
	private JTextArea textArea;
	private JPanel panelData;
	private JLabel lblLatitude;
	private JLabel lblLongitude;
	private JLabel lblHeight;
	protected JTextField textFieldLatitude;
	protected JTextField textFieldLongitude;
	protected JTextField textFieldHeight;
	private JLabel lblRoll;
	private JLabel lblPitch;
	private JLabel lblHeading;
	protected JTextField textFieldRoll;
	protected JTextField textFieldPitch;
	protected JTextField textFieldHeading;
	private JPanel panelSerial;
	private JPanel panelNetwork;
	private JTextField textHostname;
	private JTextField textTcpPort;
	private JButton btnNetworkConnect;

	// Data Connections
	private SerialPort serialPort;
	protected Boolean serialConnected = false;
	private Boolean serialPortsAvailable = false;
	private NetworkConnection networkConnection = null;

	/*
	 * Example code for sending packets
	 * 
	 * ANPacket packet = new ANPacket(1, ANPacket.PACKET_ID_REQUEST);
	 * packet.data[0] = ANPacket.PACKET_ID_BOOT_MODE;
	 * transmitPacket(packet);
	 * 
	 * The above example sends a packet of length 1 requesting the device boot
	 * mode packet.
	 */

	/**
	 * Launch the application.
	 */
	public static void main(String[] args)
	{
		EventQueue.invokeLater(new Runnable()
		{
			public void run()
			{
				try
				{
					Main window = new Main();
					window.frmAnppExample.setVisible(true);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public Main()
	{
		initialize();
	}

	public void transmitPacket(ANPacket packet)
	{
		if (networkConnection != null)
		{
			networkConnection.transmitData(ANPacketDecoder.packetEncode(packet));
		}
		else if (serialConnected)
		{
			byte[] data = ANPacketDecoder.packetEncode(packet);
			if(serialPort.writeBytes(data, data.length) != data.length)
			{
				serialPortClose();
			}
		}
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize()
	{
		// this sets swing to use the native look and feel, much more
		// aesthetically pleasing
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e)
		{
		}

		// all of the swing components and layout management is best edited
		// using the windowbuilder GUI
		frmAnppExample = new JFrame();
		frmAnppExample.setTitle("ANPP Example");
		frmAnppExample.setBounds(100, 100, 820, 630);
		frmAnppExample.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 10, 0, 0, 90, 5, 0 };
		gridBagLayout.rowHeights = new int[] { 10, 0, 0, 0, 5, 0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, 1.0, 0.0, 0.0, 0.0 };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 1.0, 0.0, 0.0 };
		frmAnppExample.getContentPane().setLayout(gridBagLayout);

		panelSerial = new JPanel();
		panelSerial.setBorder(new TitledBorder(null, "Serial Port Connection", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		GridBagConstraints gbc_panelSerial = new GridBagConstraints();
		gbc_panelSerial.gridwidth = 3;
		gbc_panelSerial.insets = new Insets(5, 5, 5, 5);
		gbc_panelSerial.fill = GridBagConstraints.BOTH;
		gbc_panelSerial.gridx = 1;
		gbc_panelSerial.gridy = 0;
		frmAnppExample.getContentPane().add(panelSerial, gbc_panelSerial);
		GridBagLayout gbl_panelSerial = new GridBagLayout();
		gbl_panelSerial.columnWidths = new int[] { 0, 0, 0, 0 };
		gbl_panelSerial.rowHeights = new int[] { 0, 0 };
		gbl_panelSerial.columnWeights = new double[] { 1.0, 0.0, 0.0, 0.0 };
		gbl_panelSerial.rowWeights = new double[] { 0.0, 0.0 };
		panelSerial.setLayout(gbl_panelSerial);

		comboSerialPort = new JComboBox<String>();
		GridBagConstraints gbc_comboSerialPort = new GridBagConstraints();
		gbc_comboSerialPort.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboSerialPort.insets = new Insets(5, 5, 5, 5);
		gbc_comboSerialPort.gridx = 0;
		gbc_comboSerialPort.gridy = 0;
		panelSerial.add(comboSerialPort, gbc_comboSerialPort);
		comboSerialPort.setEnabled(false);
		comboSerialPort.setModel(new DefaultComboBoxModel<String>(new String[] { "No Serial Ports" }));

		comboBaudRate = new JComboBox<String>();
		GridBagConstraints gbc_comboBaudRate = new GridBagConstraints();
		gbc_comboBaudRate.insets = new Insets(5, 5, 5, 5);
		gbc_comboBaudRate.gridx = 1;
		gbc_comboBaudRate.gridy = 0;
		panelSerial.add(comboBaudRate, gbc_comboBaudRate);
		comboBaudRate.setModel(new DefaultComboBoxModel<String>(new String[] { "1200", "2400", "4800", "9600", "19200", "38400", "57600", "115200", "230400", "460800", "500000", "921600",
			"1000000", "2000000", "4000000" }));
		comboBaudRate.setSelectedIndex(7);

		btnSerialConnect = new JButton("Connect");
		GridBagConstraints gbc_btnSerialConnect = new GridBagConstraints();
		gbc_btnSerialConnect.insets = new Insets(5, 5, 5, 5);
		gbc_btnSerialConnect.gridx = 2;
		gbc_btnSerialConnect.gridy = 0;
		panelSerial.add(btnSerialConnect, gbc_btnSerialConnect);
		btnSerialConnect.setEnabled(false);
		btnSerialConnect.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				// if the serial port is not connected, this is a connect action
				// if the serial port is connected, this is a disconnect action
				if (!serialConnected)
				{
					try
					{
						serialPort = SerialPort.getCommPort((String) comboSerialPort.getSelectedItem());
						serialPort.setComPortParameters(Integer.parseInt((String) comboBaudRate.getSelectedItem()), 8, SerialPort.ONE_STOP_BIT, SerialPort.NO_PARITY);
						serialPort.openPort(0);

						// add an event listener, which is the subclass at the
						// bottom of this file
						serialPort.addDataListener(new SerialReader());

						// change the button to be disconnect once we are
						// connected
						btnSerialConnect.setText("Disconnect");
						btnNetworkConnect.setEnabled(false);
						serialConnected = true;
						comboSerialPort.setEnabled(false);
						comboBaudRate.setEnabled(false);
					}
					catch (Exception exception)
					{
						System.err.println(exception.toString());
						serialPortClose();
					}
				}
				else
				{
					serialPortClose();
				}
			}
		});

		panelNetwork = new JPanel();
		panelNetwork.setBorder(new TitledBorder(null, "Network Connection", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		GridBagConstraints gbc_panelNetwork = new GridBagConstraints();
		gbc_panelNetwork.gridwidth = 3;
		gbc_panelNetwork.insets = new Insets(5, 5, 5, 5);
		gbc_panelNetwork.fill = GridBagConstraints.BOTH;
		gbc_panelNetwork.gridx = 1;
		gbc_panelNetwork.gridy = 1;
		frmAnppExample.getContentPane().add(panelNetwork, gbc_panelNetwork);
		GridBagLayout gbl_panelNetwork = new GridBagLayout();
		gbl_panelNetwork.columnWidths = new int[] { 0, 0, 0, 0 };
		gbl_panelNetwork.rowHeights = new int[] { 0, 0 };
		gbl_panelNetwork.columnWeights = new double[] { 1.0, 1.0, 0.0, 0.0 };
		gbl_panelNetwork.rowWeights = new double[] { 0.0, 0.0 };
		panelNetwork.setLayout(gbl_panelNetwork);

		textHostname = new JTextField();
		textHostname.setText("192.168.1.4");
		GridBagConstraints gbc_textHostname = new GridBagConstraints();
		gbc_textHostname.insets = new Insets(5, 5, 5, 5);
		gbc_textHostname.fill = GridBagConstraints.HORIZONTAL;
		gbc_textHostname.gridx = 0;
		gbc_textHostname.gridy = 0;
		panelNetwork.add(textHostname, gbc_textHostname);
		textHostname.setColumns(10);

		textTcpPort = new JTextField();
		textTcpPort.setText("16718");
		GridBagConstraints gbc_textTcpPort = new GridBagConstraints();
		gbc_textTcpPort.insets = new Insets(5, 5, 5, 5);
		gbc_textTcpPort.fill = GridBagConstraints.HORIZONTAL;
		gbc_textTcpPort.gridx = 1;
		gbc_textTcpPort.gridy = 0;
		panelNetwork.add(textTcpPort, gbc_textTcpPort);
		textTcpPort.setColumns(10);

		btnNetworkConnect = new JButton("Connect");
		btnNetworkConnect.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				if (networkConnection == null)
				{
					try
					{
						networkConnection = new NetworkConnection(InetAddress.getByName(textHostname.getText()), Integer.parseInt(textTcpPort.getText()));
						btnNetworkConnect.setText("Disconnect");
						btnSerialConnect.setEnabled(false);
						textHostname.setEnabled(false);
						textTcpPort.setEnabled(false);
						serialConnected = true; // this stops the serial port scanner
					}
					catch (Exception e)
					{
						e.printStackTrace();
						networkConnection = null;
					}
				}
				else
				{
					networkConnection.stop();
				}
			}
		});
		GridBagConstraints gbc_btnNetworkConnect = new GridBagConstraints();
		gbc_btnNetworkConnect.insets = new Insets(5, 5, 5, 5);
		gbc_btnNetworkConnect.gridx = 2;
		gbc_btnNetworkConnect.gridy = 0;
		panelNetwork.add(btnNetworkConnect, gbc_btnNetworkConnect);

		panelData = new JPanel();
		panelData.setBorder(new TitledBorder(null, "Data", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		GridBagConstraints gbc_panelData = new GridBagConstraints();
		gbc_panelData.gridwidth = 3;
		gbc_panelData.insets = new Insets(0, 0, 5, 5);
		gbc_panelData.fill = GridBagConstraints.BOTH;
		gbc_panelData.gridx = 1;
		gbc_panelData.gridy = 2;
		frmAnppExample.getContentPane().add(panelData, gbc_panelData);
		GridBagLayout gbl_panelData = new GridBagLayout();
		gbl_panelData.columnWidths = new int[] { 10, 0, 0, 5, 0, 5, 0, 0 };
		gbl_panelData.rowHeights = new int[] { 5, 0, 0, 0, 5, 0 };
		gbl_panelData.columnWeights = new double[] { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
		gbl_panelData.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
		panelData.setLayout(gbl_panelData);

		lblLatitude = new JLabel("Latitude:");
		GridBagConstraints gbc_lblLatitude = new GridBagConstraints();
		gbc_lblLatitude.anchor = GridBagConstraints.EAST;
		gbc_lblLatitude.insets = new Insets(0, 0, 5, 5);
		gbc_lblLatitude.gridx = 1;
		gbc_lblLatitude.gridy = 1;
		panelData.add(lblLatitude, gbc_lblLatitude);

		textFieldLatitude = new JTextField();
		GridBagConstraints gbc_textFieldLatitude = new GridBagConstraints();
		gbc_textFieldLatitude.insets = new Insets(0, 0, 5, 5);
		gbc_textFieldLatitude.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldLatitude.gridx = 2;
		gbc_textFieldLatitude.gridy = 1;
		panelData.add(textFieldLatitude, gbc_textFieldLatitude);
		textFieldLatitude.setColumns(15);

		lblRoll = new JLabel("Roll:");
		GridBagConstraints gbc_lblRoll = new GridBagConstraints();
		gbc_lblRoll.anchor = GridBagConstraints.EAST;
		gbc_lblRoll.insets = new Insets(0, 0, 5, 5);
		gbc_lblRoll.gridx = 4;
		gbc_lblRoll.gridy = 1;
		panelData.add(lblRoll, gbc_lblRoll);

		textFieldRoll = new JTextField();
		GridBagConstraints gbc_textFieldRoll = new GridBagConstraints();
		gbc_textFieldRoll.insets = new Insets(0, 0, 5, 5);
		gbc_textFieldRoll.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldRoll.gridx = 5;
		gbc_textFieldRoll.gridy = 1;
		panelData.add(textFieldRoll, gbc_textFieldRoll);
		textFieldRoll.setColumns(15);

		lblLongitude = new JLabel("Longitude:");
		GridBagConstraints gbc_lblLongitude = new GridBagConstraints();
		gbc_lblLongitude.anchor = GridBagConstraints.EAST;
		gbc_lblLongitude.insets = new Insets(0, 0, 5, 5);
		gbc_lblLongitude.gridx = 1;
		gbc_lblLongitude.gridy = 2;
		panelData.add(lblLongitude, gbc_lblLongitude);

		textFieldLongitude = new JTextField();
		GridBagConstraints gbc_textFieldLongitude = new GridBagConstraints();
		gbc_textFieldLongitude.insets = new Insets(0, 0, 5, 5);
		gbc_textFieldLongitude.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldLongitude.gridx = 2;
		gbc_textFieldLongitude.gridy = 2;
		panelData.add(textFieldLongitude, gbc_textFieldLongitude);
		textFieldLongitude.setColumns(15);

		lblPitch = new JLabel("Pitch:");
		GridBagConstraints gbc_lblPitch = new GridBagConstraints();
		gbc_lblPitch.anchor = GridBagConstraints.EAST;
		gbc_lblPitch.insets = new Insets(0, 0, 5, 5);
		gbc_lblPitch.gridx = 4;
		gbc_lblPitch.gridy = 2;
		panelData.add(lblPitch, gbc_lblPitch);

		textFieldPitch = new JTextField();
		GridBagConstraints gbc_textFieldPitch = new GridBagConstraints();
		gbc_textFieldPitch.insets = new Insets(0, 0, 5, 5);
		gbc_textFieldPitch.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldPitch.gridx = 5;
		gbc_textFieldPitch.gridy = 2;
		panelData.add(textFieldPitch, gbc_textFieldPitch);
		textFieldPitch.setColumns(15);

		lblHeight = new JLabel("Height:");
		GridBagConstraints gbc_lblHeight = new GridBagConstraints();
		gbc_lblHeight.anchor = GridBagConstraints.EAST;
		gbc_lblHeight.insets = new Insets(0, 0, 5, 5);
		gbc_lblHeight.gridx = 1;
		gbc_lblHeight.gridy = 3;
		panelData.add(lblHeight, gbc_lblHeight);

		textFieldHeight = new JTextField();
		GridBagConstraints gbc_textFieldHeight = new GridBagConstraints();
		gbc_textFieldHeight.insets = new Insets(0, 0, 5, 5);
		gbc_textFieldHeight.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldHeight.gridx = 2;
		gbc_textFieldHeight.gridy = 3;
		panelData.add(textFieldHeight, gbc_textFieldHeight);
		textFieldHeight.setColumns(15);

		lblHeading = new JLabel("Heading:");
		GridBagConstraints gbc_lblYaw = new GridBagConstraints();
		gbc_lblYaw.anchor = GridBagConstraints.EAST;
		gbc_lblYaw.insets = new Insets(0, 0, 5, 5);
		gbc_lblYaw.gridx = 4;
		gbc_lblYaw.gridy = 3;
		panelData.add(lblHeading, gbc_lblYaw);

		textFieldHeading = new JTextField();
		GridBagConstraints gbc_textFieldYaw = new GridBagConstraints();
		gbc_textFieldYaw.insets = new Insets(0, 0, 5, 5);
		gbc_textFieldYaw.fill = GridBagConstraints.HORIZONTAL;
		gbc_textFieldYaw.gridx = 5;
		gbc_textFieldYaw.gridy = 3;
		panelData.add(textFieldHeading, gbc_textFieldYaw);
		textFieldHeading.setColumns(15);

		textArea = new JTextArea();
		JScrollPane scrollPane = new JScrollPane(textArea);
		GridBagConstraints gbc_textArea = new GridBagConstraints();
		gbc_textArea.gridwidth = 3;
		gbc_textArea.insets = new Insets(0, 0, 5, 5);
		gbc_textArea.fill = GridBagConstraints.BOTH;
		gbc_textArea.gridx = 1;
		gbc_textArea.gridy = 3;
		frmAnppExample.getContentPane().add(scrollPane, gbc_textArea);

		// create a thread to update the available serials ports once a second
		// it is done in a new thread because under windows it can block for up
		// to 10 seconds
		Thread portScannerThread = new Thread(new PortScanner());
		portScannerThread.start();
	}

	public class PortScanner implements Runnable
	{
		public void run()
		{
			while (true)
			{
				if (!serialConnected)
				{
					SwingUtilities.invokeLater(new Runnable()
					{
						public void run()
						{
							if (!serialConnected)
							{
								SerialPort[] serialPorts = SerialPort.getCommPorts();
								int comboLength = comboSerialPort.getItemCount();
								if (!serialPortsAvailable) comboLength = 0;
								Boolean portsChanged = serialPorts.length != comboLength;
								if (!portsChanged)
								{
									for (int i = 0; i < serialPorts.length; i++)
									{
										if (serialPorts[i].getSystemPortName().compareTo(comboSerialPort.getItemAt(i)) != 0)
										{
											portsChanged = true;
											break;
										}
									}
								}
								if (portsChanged)
								{
									comboSerialPort.removeAllItems();
									for (int i = 0; i < serialPorts.length; i++)
									{
										comboSerialPort.addItem(serialPorts[i].getSystemPortName());
									}
									if (serialPorts.length == 0)
									{
										comboSerialPort.addItem("No Serial Ports");
										comboSerialPort.setSelectedIndex(0);
										comboSerialPort.setEnabled(false);
										btnSerialConnect.setEnabled(false);
										serialPortsAvailable = false;
									}
									else
									{
										comboSerialPort.setSelectedIndex(0);
										comboSerialPort.setEnabled(true);
										btnSerialConnect.setEnabled(true);
										serialPortsAvailable = true;
									}
								}
							}
						}
					});
				}
				try
				{
					Thread.sleep(1000);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	private void serialPortClose()
	{
		try
		{
			serialPort.removeDataListener();
			serialPort.closePort();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		btnSerialConnect.setText("Connect");
		btnNetworkConnect.setEnabled(true);
		serialConnected = false;
		comboSerialPort.setEnabled(true);
		comboBaudRate.setEnabled(true);
	}

	public void processPacket(ANPacket packet)
	{
		switch (packet.id)
		{
			case ANPacket.PACKET_ID_RAW_SENSORS:
				if (packet.length == ANPacket28.LENGTH)
				{
					final ANPacket28 anPacket28 = new ANPacket28(packet);

					SwingUtilities.invokeLater(new Runnable()
					{
						public void run()
						{
							textArea.append("Received raw sensors packet. X accelerometer value: " + anPacket28.accelerometers[0] + "\n");
							textArea.setCaretPosition(textArea.getDocument().getLength());
						}
					});
				}
				break;
			case ANPacket.PACKET_ID_SYSTEM_STATE:
				if (packet.length == ANPacket20.LENGTH)
				{
					final ANPacket20 anPacket20 = new ANPacket20(packet);

					SwingUtilities.invokeLater(new Runnable()
					{
						public void run()
						{
							textArea.append("Received system state packet with timestamp " + anPacket20.time + "\n");
							textArea.setCaretPosition(textArea.getDocument().getLength());
							textFieldLatitude.setText(Double.toString(anPacket20.position[0] * 180 / Math.PI));
							textFieldLongitude.setText(Double.toString(anPacket20.position[1] * 180 / Math.PI));
							textFieldHeight.setText(Double.toString(anPacket20.position[2]));
							textFieldRoll.setText(Double.toString(anPacket20.orientation[0] * 180 / Math.PI));
							textFieldPitch.setText(Double.toString(anPacket20.orientation[1] * 180 / Math.PI));
							textFieldHeading.setText(Double.toString(anPacket20.orientation[2] * 180 / Math.PI));

						}
					});
				}
				break;
			default:
				break;
		}
	}

	public class NetworkConnection implements Runnable
	{
		private Socket tcpSocket;
		private OutputStream tcpOutputStream;

		private InetAddress clientAddress;
		private int port;
		private Boolean connected = false;
		private Thread receiveThread;
		private ANPacketDecoder packetDecoder;

		public NetworkConnection(InetAddress clientAddress, int port) throws IOException
		{
			packetDecoder = new ANPacketDecoder();
			this.port = port;
			this.clientAddress = clientAddress;
			receiveThread = new Thread(this);
			receiveThread.start();
		}

		public void stop()
		{
			receiveThread.interrupt();
			try
			{
				tcpSocket.close();
				networkConnection = null;
				serialConnected = false;
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						btnNetworkConnect.setText("Connect");
						btnSerialConnect.setEnabled(true);
						textHostname.setEnabled(true);
						textTcpPort.setEnabled(true);
					}
				});
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}

		public void transmitData(byte[] data)
		{
			if (connected)
			{
				try
				{
					tcpOutputStream.write(data);
				}
				catch (IOException e)
				{
					// Socket disconnected
					stop();
				}
			}
		}

		@Override
		public void run()
		{
			InputStream tcpInputStream = null;
			while (!Thread.currentThread().isInterrupted())
			{
				if (!connected)
				{
					try
					{
						tcpSocket = new Socket();
						tcpSocket.connect(new InetSocketAddress(clientAddress, port), 3000);
						tcpSocket.setSoTimeout(15000);
						tcpInputStream = tcpSocket.getInputStream();
						tcpOutputStream = tcpSocket.getOutputStream();
						connected = true;
					}
					catch (Exception e)
					{
						// Error connecting
						stop();
						break;
					}
				}
				else
				{
					try
					{
						int bytesReceived = tcpInputStream.read(packetDecoder.buffer, packetDecoder.bufferLength, packetDecoder.bufferSpace());
						if (bytesReceived > 0)
						{
							packetDecoder.bufferLength += bytesReceived;
							ANPacket packet = null;
							while ((packet = packetDecoder.packetDecode()) != null)
							{
								processPacket(packet);
							}
						}
						else
						{
							// Socket disconnected
							stop();
							break;
						}
					}
					catch (Exception e1)
					{
						// Socket disconnected
						stop();
						break;
					}
				}
			}
		}
	}

	/*
	 * This class is the event listener for received serial data
	 * Whenever serial data is received the serialEvent is called
	 * which adds the received data to the decoders buffer and then
	 * checks whether any packets can be decoded.
	 */
	public class SerialReader implements SerialPortDataListener
	{
		ANPacketDecoder packetDecoder;

		public SerialReader()
		{
			packetDecoder = new ANPacketDecoder();
		}
		
		@Override
		public int getListeningEvents()
		{
			return SerialPort.LISTENING_EVENT_DATA_AVAILABLE;
		}

		public void serialEvent(SerialPortEvent event)
		{
			if (event.getEventType() == SerialPort.LISTENING_EVENT_DATA_AVAILABLE)
			{
				try
				{
					packetDecoder.bufferLength += serialPort.readBytes(packetDecoder.buffer, packetDecoder.buffer.length - packetDecoder.bufferLength, packetDecoder.bufferLength);
					ANPacket packet = null;
					while ((packet = packetDecoder.packetDecode()) != null)
					{
						processPacket(packet);
					}
				}
				catch (Exception exception)
				{
					System.err.println(exception.toString());
				}
			}
		}
	}
}
